library PartNameCom;

{ Important note about DLL memory management: ShareMem must be the
  first unit in your library's USES clause AND your project's (select
  Project-View Source) USES clause if your DLL exports any procedures or
  functions that pass strings as parameters or function results. This
  applies to all strings passed to and from your DLL--even those that
  are nested in records and classes. ShareMem is the interface unit to
  the BORLNDMM.DLL shared memory manager, which must be deployed along
  with your DLL. To avoid using BORLNDMM.DLL, pass string information
  using PChar or ShortString parameters. }

uses
  SysUtils,
  Classes;

{$R *.res}

// a very basic logging. Should be disabled in final version of plugin
procedure log(logString:string);
const LOGNAME:string='c:\tmp\partnamecom.log';
const ENABLED:boolean=false;
var logFile:TextFile;
begin
  if ENABLED then
  begin
    {$IOChecks OFF}
    AssignFile(logFile, LOGNAME);
    if FileExists(LOGNAME) then
      Append(logFile)
    else
      Rewrite(logFile);
    Writeln(logFile, logString);
    CloseFile(logFile);
    {$IOCHECKS ON}
  end;
end;


{used to split the filenames into single words}
procedure Split(var res:TStringList; const fText: String; const fSep: Char; fTrim: Boolean=false; fQuotes: Boolean=false);
var vI: Integer;
    vBuffer: String;
    vOn: Boolean;
begin
  res:=TStringList.Create;
  vBuffer:='';
  vOn:=true;
  for vI:=1 to Length(fText) do
  begin
    if (fQuotes and(fText[vI]=fSep)and vOn)or(Not(fQuotes) and (fText[vI]=fSep)) then
    begin
      if fTrim then vBuffer:=Trim(vBuffer);
      if vBuffer='' then vBuffer:=fSep; // !!! sonst luft z.B. split(',**',',') auf einen Hammer...
      if vBuffer[1]=fSep then
        vBuffer:=Copy(vBuffer,2,Length(vBuffer));
      Res.Add(vBuffer);
      vBuffer:='';
    end;
    if fQuotes then
    begin
      if fText[vI]='"' then
      begin
        vOn:=Not(vOn);
        Continue;
      end;
      if (fText[vI]<>fSep)or((fText[vI]=fSep)and(vOn=false)) then
        vBuffer:=vBuffer+fText[vI];
    end else
      if fText[vI]<>fSep then
        vBuffer:=vBuffer+fText[vI];
  end;
  if vBuffer<>'' then
  begin
    if fTrim then vBuffer:=Trim(vBuffer);
    Res.Add(vBuffer);
  end;
end;


function refineName(filename:string):string;
var extPos:integer;
begin
  log('unprocessed: ' + filename);
  // remove extension
  extPos:=pos(ExtractFileExt(fileName),fileName);
  if extPos>0 then
    delete(fileName,extPos,MaxInt);
  log('no extension: '+filename);
  // replace _ with a space
  while pos('_',fileName)>0 do
    fileName[pos('_',fileName)]:=' ';
  // same for -
  while pos('-',fileName)>0 do
    fileName[pos('-',fileName)]:=' ';
  log('cleaned: '+filename);
  result:=filename;
end;


{****************************************************
 * Start of DupKiller Plugin API functions          *
 ****************************************************}

{This function must return TRUE (always!). This is for DupKiller to
find its native libraries.}
function DupKillerPlugin:boolean;
begin
  DupKillerPlugin := true;
end;


{This function returns information about plugin using "value".
For example:
GetPluginInfo('name') -- returns name of plugin (e.g, 'ApproCom');
GetPluginInfo('author') -- returns name of plugin (e.g, 'Oleksandr
Roslov');
GetPluginInfo('version') -- returns plugin version (e.g, '0.1');
GetPluginInfo('comparer') -- returns '1' if plugin compares files
during scan process, or '0' if not;
GetPluginInfo('viewer') -- returns '1' if plugin is a file viewer,
or '0' if not (this feature is NOT supported yet, so it must return
'0' now);
GetPluginInfo('tbc') -- returns '1' if plugin uses true byte
comparison during scan process, or '0' if not;
GetPluginInfo('description') -- returns description of plugin
(e.g., 'Compares files using not whole contents but random
samples.').
Any other value must return an empty string ('').}
function GetPluginInfo(value:PChar):PChar;
begin
  if value='name' then
    GetPluginInfo:='PartNameCom'
  else if value='author' then
    GetPluginInfo:='Sven Haul'
  else if value='version' then
    GetPluginInfo:='0.3'
  else if value='comparer' then
    GetPluginInfo:='1'
  else if value='viewer' then
    GetPluginInfo:='0'
  else if value='tbc' then
    GetPluginInfo:='0'
  else if value='description' then
    GetPluginInfo:='Compares filenames by exactly matching parts of it. ' +
      'This is especially useful for renamed mp3 files, where e.g. the title still '+
      'is the same, but at another position in the filename.'
  else
    GetPluginInfo:='';
end;


{This function compares file. It must return TRUE if files are dupes,
or FALSE in other case.
f1 and f2 -- full paths (with filenames) to file to be compared.}
function PluginCompare(f1,f2:PChar):boolean;
var wordsInF1, wordsInF2, wordsToScan, wordsToCompare:TStringList;
var currentWord:string;
var wordsFound:integer;
var cleanName:string;
  i:integer;
begin
  log('Compare '+f1+' to '+f2);
  {the comparison is based on same words in the filename:
  Only the filename without drive and path is split
  into individual words. The lesser amount of words (i.e.
  shorter name) is compared against the other name. If all
  words of the shorter name are present, the file is a
  duplicate.}

  // get the filename, replace _ with space, and split it
  cleanName:=refineName(ExtractFileName(f1));
  log('splitting "'+cleanName+'"');
//  wordsInF1:=Split(cleanName,' ',true,false);
  Split(wordsInF1, cleanName,' ',true,false);
  log('split to '+wordsInF1.CommaText);
  cleanName:=refineName(ExtractFileName(f2));
  log('splitting "'+cleanName+'"');
//  wordsInF2:=Split(cleanName,' ',true,false);
  Split(wordsInF2, cleanName,' ',true,false);
  log('split to '+wordsInF2.CommaText);
  // set the smaller list to scan through
  if wordsInF1.count<wordsInF2.count then
  begin
    wordsToScan:=wordsInF1;
    wordsToCompare:=wordsInF2;
  end
  else
  begin
    wordsToScan:=wordsInF2;
    wordsToCompare:=wordsInF1;
  end;
  if wordsToScan.Count >0 then  // safety catch to prevent lockup
  begin
    wordsToScan.CaseSensitive := false;   // for comparison
    // go through the words in the smaller list
    // and see how many are in the bigger list
    wordsFound:=0;
    for i:=0 to wordsToScan.Count-1 do
//    for currentWord  in wordsToScan  do
    begin
      currentWord:=wordsToScan.Strings[i];
      log('current word is '+currentWord);
      if (wordsToCompare.IndexOf(currentWord )>-1) then
      begin
        inc(wordsFound);
        log('found in other list!');
      end;
    end;
    log('found '+IntToStr(wordsFound)
      +' of '
      +IntToStr(wordsToScan.count)
      +' words, '
      +FloatToStr((wordsFound/wordsToScan.Count)*100)
      +'% match');
    // more than 90% of the words found: duplicate
    result:=((wordsFound/wordsToScan.Count)>0.9);
  end
  else
  begin
    result:=false;  //
  end;
  log('result:'+BoolToStr(result) );
  log('');

  wordsInF1.Free;
  wordsInF2.Free;
{  wordsToScan.Free;
  wordsToCompare.Free;}
end;


{This function shows plugin properties window. It is not necessary
function.}
procedure ShowPluginProperties;
begin
{No need to make this function public if it does nothing.
In this case the 'Settings' button will be grayed.}
end;


{This function returns minimal version of DupKiller supporting this plugin}
function GetDupKillerVersion:PChar; export;
begin
  result:='0.8.2';
end;



{make the API public}
exports DupKillerPlugin,GetPluginInfo,PluginCompare,GetDupKillerVersion; //,ShowPluginProperties;


begin
end.
